@@ -1,16 +1,16 @@
 # Copyright (c) 2009, Peter Sagerson
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 # - Redistributions of source code must retain the above copyright notice, this
 # list of conditions and the following disclaimer.
-# 
+#
 # - Redistributions in binary form must reproduce the above copyright notice,
 # this list of conditions and the following disclaimer in the documentation
 # and/or other materials provided with the distribution.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -45,7 +45,7 @@ class _LDAPConfig(object):
     """
     ldap = None
     logger = None
-    
+
     def get_ldap(cls):
         """
         Returns the ldap module. The unit test harness will assign a mock object
@@ -63,9 +63,9 @@ class _LDAPConfig(object):
             except ImportError:
                 from django_auth_ldap import dn
                 ldap.dn = dn
-            
+
             cls.ldap = ldap
-        
+
         return cls.ldap
     get_ldap = classmethod(get_ldap)
 
@@ -105,7 +105,7 @@ class LDAPSearch(object):
         self.scope = scope
         self.filterstr = filterstr
         self.ldap = _LDAPConfig.get_ldap()
-    
+
     def search_with_additional_terms(self, term_dict, escape=True):
         """
         Returns a new search object with additional search terms and-ed to the
@@ -113,16 +113,16 @@ class LDAPSearch(object):
         you don't want the values escaped, pass escape=False.
         """
         term_strings = [self.filterstr]
-        
+
         for name, value in term_dict.iteritems():
             if escape:
                 value = self.ldap.filter.escape_filter_chars(value)
             term_strings.append(u'(%s=%s)' % (name, value))
-        
+
         filterstr = u'(&%s)' % ''.join(term_strings)
-        
+
         return self.__class__(self.base_dn, self.scope, filterstr)
-    
+
     def search_with_additional_term_string(self, filterstr):
         """
         Returns a new search object with filterstr and-ed to the original filter
@@ -130,14 +130,14 @@ class LDAPSearch(object):
         string.
         """
         filterstr = u'(&%s%s)' % (self.filterstr, filterstr)
-        
+
         return self.__class__(self.base_dn, self.scope, filterstr)
-    
+
     def execute(self, connection, filterargs=()):
         """
         Executes the search on the given connection (an LDAPObject). filterargs
         is an object that will be used for expansion of the filter string.
-        
+
         The python-ldap library returns utf8-encoded strings. For the sake of
         sanity, this method will decode all result strings and return them as
         Unicode.
@@ -160,7 +160,7 @@ class LDAPSearch(object):
             results = []
             logger.error(u"search_s('%s', %d, '%s') raised %s" %
                 (self.base_dn, self.scope, filterstr, pprint.pformat(e)))
-        
+
         return results
 
 
@@ -172,7 +172,7 @@ class _DeepStringCoder(object):
     """
     def __init__(self, encoding):
         self.encoding = encoding
-    
+
     def decode(self, value):
         try:
             if isinstance(value, str):
@@ -185,14 +185,21 @@ class _DeepStringCoder(object):
                 value = self._decode_dict(value)
         except UnicodeDecodeError:
             pass
-        
+
         return value
-    
+
     def _decode_list(self, value):
         return [self.decode(v) for v in value]
-    
+
     def _decode_dict(self, value):
-        return dict([(self.decode(k), self.decode(v)) for k,v in value.iteritems()])
+        # python-ldap has a special case-insensitive dictionary class, so we
+        # need to respect that.
+        decoded = value.__class__()
+
+        for k, v in value.iteritems():
+            decoded[self.decode(k)] = self.decode(v)
+
+        return decoded
 
 
 class LDAPGroupType(object):
@@ -202,10 +209,10 @@ class LDAPGroupType(object):
     a concrete subclass for each grouping mechanism. Clients may subclass this
     if they have a group mechanism that is not handled by a built-in
     implementation.
-    
+
     name_attr is the name of the LDAP attribute from which we will take the
     Django group name.
-    
+
     Subclasses in this file must use self.ldap to access the python-ldap module.
     This will be a mock object during unit tests.
     """
@@ -221,22 +228,22 @@ class LDAPGroupType(object):
         will apply additional filters to group_search and return the results of
         the search. ldap_user represents the user and has the following three
         properties:
-        
+
         dn: the distinguished name
         attrs: a dictionary of LDAP attributes (with lists of values)
         connection: an LDAPObject that has been bound with credentials
-        
+
         This is the primitive method in the API and must be implemented.
         """
         return []
-    
+
     def is_member(self, ldap_user, group_dn):
         """
         This method is an optimization for determining group membership without
         loading all of the user's groups. Subclasses that are able to do this
         may return True or False. ldap_user is as above. group_dn is the
         distinguished name of the group in question.
-        
+
         The base implementation returns None, which means we don't have enough
         information. The caller will have to call user_groups() instead and look
         for group_dn in the results.
@@ -248,7 +255,7 @@ class LDAPGroupType(object):
         Given the (DN, attrs) 2-tuple of an LDAP group, this returns the name of
         the Django group. This may return None to indicate that a particular
         LDAP group has no corresponding Django group.
-        
+
         The base implementation returns the value of the cn attribute, or
         whichever attribute was given to __init__ in the name_attr
         parameter.
@@ -257,7 +264,7 @@ class LDAPGroupType(object):
             name = group_info[1][self.name_attr][0]
         except (KeyError, IndexError):
             name = None
-        
+
         return name
 
 
@@ -271,21 +278,21 @@ class PosixGroupType(LDAPGroupType):
         user as a member.
         """
         groups = []
-        
+
         try:
             user_uid = ldap_user.attrs['uid'][0]
             user_gid = ldap_user.attrs['gidNumber'][0]
-            
+
             filterstr = u'(|(gidNumber=%s)(memberUid=%s))' % (
                 self.ldap.filter.escape_filter_chars(user_gid),
                 self.ldap.filter.escape_filter_chars(user_uid)
             )
-            
+
             search = group_search.search_with_additional_term_string(filterstr)
             groups = search.execute(ldap_user.connection)
         except (KeyError, IndexError):
             pass
-        
+
         return groups
 
     def is_member(self, ldap_user, group_dn):
@@ -302,7 +309,7 @@ class PosixGroupType(LDAPGroupType):
                 is_member = ldap_user.connection.compare_s(group_dn.encode('utf-8'), 'gidNumber', user_gid.encode('utf-8'))
         except (KeyError, IndexError):
             is_member = False
-        
+
         return is_member
 
 
@@ -316,14 +323,14 @@ class MemberDNGroupType(LDAPGroupType):
         member DNs.
         """
         self.member_attr = member_attr
-        
+
         super(MemberDNGroupType, self).__init__(name_attr)
-    
+
     def user_groups(self, ldap_user, group_search):
         search = group_search.search_with_additional_terms(
             {self.member_attr: ldap_user.dn})
         groups = search.execute(ldap_user.connection)
-        
+
         return groups
 
     def is_member(self, ldap_user, group_dn):
@@ -345,7 +352,7 @@ class NestedMemberDNGroupType(LDAPGroupType):
         self.member_attr = member_attr
 
         super(NestedMemberDNGroupType, self).__init__(name_attr)
-        
+
     def user_groups(self, ldap_user, group_search):
         """
         This searches for all of a user's groups from the bottom up. In other
@@ -356,7 +363,7 @@ class NestedMemberDNGroupType(LDAPGroupType):
         group_info_map = {} # Maps group_dn to group_info of groups we've found
         member_dn_set = set([ldap_user.dn]) # Member DNs to search with next
         handled_dn_set = set() # Member DNs that we've already searched with
-        
+
         while len(member_dn_set) > 0:
             group_infos = self.find_groups_with_any_member(member_dn_set,
                 group_search, ldap_user.connection)
@@ -367,18 +374,18 @@ class NestedMemberDNGroupType(LDAPGroupType):
             # Get ready for the next iteration. To avoid cycles, we make sure
             # never to search with the same member DN twice.
             member_dn_set = set(new_group_info_map.keys()) - handled_dn_set
-        
+
         return group_info_map.values()
-        
+
     def find_groups_with_any_member(self, member_dn_set, group_search, connection):
         terms = [
             u"(%s=%s)" % (self.member_attr, self.ldap.filter.escape_filter_chars(dn))
             for dn in member_dn_set
         ]
-        
+
         filterstr = u"(|%s)" % "".join(terms)
         search = group_search.search_with_additional_term_string(filterstr)
-        
+
         return search.execute(connection)
 
 
